#!/usr/bin/python3
# -*- coding: utf-8 -*-

"""img_proof oci provider unit tests."""

# Copyright (c) 2019 SUSE LLC. All rights reserved.
#
# This file is part of img_proof. img_proof provides an api and command line
# utilities for testing images in the Public Cloud.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import pytest

from img_proof.ipa_oci import OCICloud
from img_proof.ipa_exceptions import OCICloudException

from unittest.mock import MagicMock, patch


class TestOCIProvider(object):
    """Test OCI provider class."""

    def setup_method(self, method):
        """Set up kwargs dict."""
        self.kwargs = {
            'config': 'tests/data/config',
            'distro_name': 'SLES',
            'image_id': 'fakeimage',
            'no_default_test_dirs': True,
            'cloud_config': 'tests/oci/config',
            'test_files': ['test_image'],
            'oci_user_id':
                'ocid1.user.oc1..'
                'aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa',
            'signing_key_fingerprint':
                '00:00:00:00:00:00:00:00:00:00:00:00:00:00:00:00',
            'signing_key_file': 'tests/oci/api_key.pem',
            'tenancy':
                'ocid1.tenancy.oc1..'
                'aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa'
        }

    def test_oci_exception_required_args(self):
        """Test an exception is raised if required args missing."""
        # Test missing availability domain
        with pytest.raises(OCICloudException):
            OCICloud(**self.kwargs)

        self.kwargs['availability_domain'] = 'Omic:PHX-AD-1'

        # Test missing compartment id
        with pytest.raises(OCICloudException):
            OCICloud(**self.kwargs)

        self.kwargs['compartment_id'] = (
            'ocid1.compartment.oc1..'
            'aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa'
        )

        # Test missing ssh private key file
        with pytest.raises(OCICloudException):
            OCICloud(**self.kwargs)

    @patch('img_proof.ipa_oci.oci')
    def test_oci_init(self, mock_oci):
        """Test oci init method and config setup is successful."""
        client = MagicMock()

        mock_oci.core.ComputeClient.return_value = client
        mock_oci.core.ComputeClientCompositeOperations.return_value = client
        mock_oci.core.VirtualNetworkClient.return_value = client
        mock_oci.core.VirtualNetworkClientCompsiteOperations.return_value = client  # noqa

        self.kwargs['availability_domain'] = 'Omic:PHX-AD-1'
        self.kwargs['compartment_id'] = (
            'ocid1.compartment.oc1..'
            'aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa'
        )
        self.kwargs['ssh_private_key_file'] = 'tests/oci/api_key.pem'

        OCICloud(**self.kwargs)

    @patch.object(OCICloud, '__init__')
    def test_oci_create_internet_gateway(self, mock_init):
        """Test oci create internet gateway method."""
        mock_init.return_value = None

        client = MagicMock()
        vcn = MagicMock()
        route_table = MagicMock()

        vcn.id = (
            'ocid1.vcn.oc1..'
            'aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa'
        )
        vcn.default_route_table_id = (
            'ocid1.routeTable.oc1..'
            'aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa'
        )

        gateway = MagicMock()
        gateway.id = (
            'ocid1.gateway.oc1..'
            'aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa'
        )

        gateway_response = MagicMock()
        gateway_response.data = gateway
        client.create_internet_gateway_and_wait_for_state.return_value = gateway_response  # noqa

        route_table_response = MagicMock()
        route_table_response.data = route_table
        route_table.route_rules = []
        client.get_route_table.return_value = route_table_response

        update_table_response = MagicMock()
        update_table_response.data = []
        client.update_route_table_and_wait_for_state.return_value = update_table_response  # noqa

        cloud = OCICloud(**self.kwargs)
        cloud.vnet_composite_client = client
        cloud.vnet_client = client

        compartment_id = (
            'ocid1.compartment.oc1..'
            'aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa'
        )
        cloud._create_internet_gateway(compartment_id, vcn, 'test-name')

    @patch.object(OCICloud, '__init__')
    def test_oci_delete_internet_gateway(self, mock_init):
        """Test oci delete internet gateway method."""
        mock_init.return_value = None

        client = MagicMock()

        gateway_id = (
            'ocid1.gateway.oc1..'
            'aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa'
        )

        cloud = OCICloud(**self.kwargs)
        cloud.vnet_composite_client = client

        cloud._delete_internet_gateway(gateway_id)

    @patch('img_proof.ipa_oci.oci')
    @patch.object(OCICloud, '__init__')
    def test_oci_get_internet_gateway_by_name(self, mock_init, mock_oci):
        """Test oci get internet gateway by name method."""
        mock_init.return_value = None

        gateway = MagicMock()
        gateway.display_name = 'test-gw2'
        client = MagicMock()
        response = MagicMock()
        response.data = [gateway]

        mock_oci.pagination.list_call_get_all_results.return_value = response

        vcn_id = (
            'ocid1.vcn.oc1..'
            'aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa'
        )

        cloud = OCICloud(**self.kwargs)
        cloud.vnet_client = client
        cloud.compartment_id = (
            'ocid1.compartment.oc1..'
            'aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa'
        )

        result = cloud._get_gateway_in_vcn_by_name(vcn_id, 'test-gw2')
        assert result == gateway

    @patch.object(OCICloud, '__init__')
    def test_oci_create_vcn(self, mock_init):
        """Test oci create vcn method."""
        mock_init.return_value = None

        client = MagicMock()
        vcn = MagicMock()
        response = MagicMock()
        response.data = vcn

        client.create_vcn_and_wait_for_state.return_value = response

        cloud = OCICloud(**self.kwargs)
        cloud.compartment_id = (
            'ocid1.compartment.oc1..'
            'aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa'
        )
        cloud.vnet_composite_client = client

        result = cloud._create_vcn('vcn-123')
        assert result == vcn

    @patch.object(OCICloud, '__init__')
    def test_oci_get_vcn(self, mock_init):
        """Test oci get vcn method."""
        mock_init.return_value = None

        client = MagicMock()
        vcn = MagicMock()
        response = MagicMock()
        response.data = vcn

        client.get_vcn.return_value = response

        cloud = OCICloud(**self.kwargs)
        cloud.vnet_client = client

        vcn_id = (
            'ocid1.vcn.oc1..'
            'aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa'
        )

        result = cloud._get_vcn(vcn_id)
        assert result == vcn

    @patch('img_proof.ipa_oci.oci')
    @patch.object(OCICloud, '__init__')
    def test_oci_get_vcn_by_name(self, mock_init, mock_oci):
        """Test oci get vcn by name method."""
        mock_init.return_value = None

        client = MagicMock()
        vcn = MagicMock()
        vcn.display_name = 'vcn-123'
        response = MagicMock()
        response.data = [vcn]

        mock_oci.pagination.list_call_get_all_results.return_value = response

        cloud = OCICloud(**self.kwargs)
        cloud.compartment_id = (
            'ocid1.compartment.oc1..'
            'aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa'
        )
        cloud.vnet_client = client

        result = cloud._get_vcn_by_name('vcn-123')
        assert result == vcn

    @patch.object(OCICloud, '__init__')
    def test_oci_delete_vcn(self, mock_init):
        """Test oci delete vcn method."""
        mock_init.return_value = None

        client = MagicMock()

        cloud = OCICloud(**self.kwargs)
        cloud.vnet_composite_client = client

        vcn_id = (
            'ocid1.vcn.oc1..'
            'aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa'
        )

        cloud._delete_vcn(vcn_id)

    @patch.object(OCICloud, '__init__')
    def test_oci_create_subnet(self, mock_init):
        """Test oci create subnet method."""
        mock_init.return_value = None

        client = MagicMock()
        subnet = MagicMock()
        response = MagicMock()
        vcn = MagicMock()
        vcn.id = (
            'ocid1.vcn.oc1..'
            'aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa'
        )
        response.data = subnet

        client.create_subnet_and_wait_for_state.return_value = response

        compartment_id = (
            'ocid1.compartment.oc1..'
            'aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa'
        )

        cloud = OCICloud(**self.kwargs)
        cloud.vnet_composite_client = client

        result = cloud._create_subnet(
            compartment_id,
            'Omic:PHX-AD-1',
            vcn,
            'subnet-123'
        )
        assert result == subnet

    @patch.object(OCICloud, '__init__')
    def test_oci_get_subnet(self, mock_init):
        """Test oci get subnet method."""
        mock_init.return_value = None

        client = MagicMock()
        subnet = MagicMock()
        response = MagicMock()
        response.data = subnet

        client.get_subnet.return_value = response

        cloud = OCICloud(**self.kwargs)
        cloud.vnet_client = client

        subnet_id = (
            'ocid1.subnet.oc1..'
            'aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa'
        )

        result = cloud._get_subnet(subnet_id)
        assert result == subnet

    @patch('img_proof.ipa_oci.oci')
    @patch.object(OCICloud, '__init__')
    def test_oci_get_subnet_by_name(self, mock_init, mock_oci):
        """Test oci get subnet by name method."""
        mock_init.return_value = None

        client = MagicMock()
        subnet = MagicMock()
        subnet.display_name = 'subnet-123'
        response = MagicMock()
        response.data = [subnet]

        mock_oci.pagination.list_call_get_all_results.return_value = response

        cloud = OCICloud(**self.kwargs)
        cloud.compartment_id = (
            'ocid1.compartment.oc1..'
            'aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa'
        )
        cloud.vnet_client = client

        vcn_id = (
            'ocid1.vcn.oc1..'
            'aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa'
        )

        result = cloud._get_subnet_by_name('subnet-123', vcn_id)
        assert result == subnet

    @patch.object(OCICloud, '__init__')
    def test_oci_delete_subnet(self, mock_init):
        """Test oci delete subnet method."""
        mock_init.return_value = None

        client = MagicMock()

        cloud = OCICloud(**self.kwargs)
        cloud.vnet_composite_client = client

        subnet_id = (
            'ocid1.subnet.oc1..'
            'aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa'
        )

        cloud._delete_subnet(subnet_id)

    @patch.object(OCICloud, '__init__')
    def test_oci_clear_route_rules(self, mock_init):
        """Test oci clear route rules method."""
        mock_init.return_value = None

        client = MagicMock()
        vcn = MagicMock()

        cloud = OCICloud(**self.kwargs)
        cloud.vnet_composite_client = client

        vcn.default_route_table_id = (
            'ocid1.routeTable.oc1..'
            'aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa'
        )

        cloud._clear_route_rules(vcn)

    @patch.object(OCICloud, '__init__')
    def test_oci_is_instance_running(self, mock_init):
        """Test oci is instance running method."""
        mock_init.return_value = None

        client = MagicMock()
        instance = MagicMock()
        instance.lifecycle_state = 'RUNNING'
        response = MagicMock()
        response.data = instance
        client.get_instance.return_value = response

        cloud = OCICloud(**self.kwargs)
        cloud.running_instance_id = 'instance-123'
        cloud.compute_client = client

        result = cloud._is_instance_running()
        assert result

    @patch('img_proof.ipa_oci.oci')
    @patch.object(OCICloud, '__init__')
    def test_oci_get_vnic_attachments(self, mock_init, mock_oci):
        """Test oci get vnic attachments method."""
        mock_init.return_value = None

        client = MagicMock()
        vnic = MagicMock()
        response = MagicMock()
        response.data = [vnic]

        mock_oci.pagination.list_call_get_all_results.return_value = response

        cloud = OCICloud(**self.kwargs)
        cloud.compute_client = client

        compartment_id = (
            'ocid1.compartment.oc1..'
            'aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa'
        )
        instance_id = (
            'ocid1.instance.oc1..'
            'aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa'
        )

        cloud._get_vnic_attachments(compartment_id, instance_id)

    @patch('img_proof.ipa_utils.generate_public_ssh_key')
    @patch.object(OCICloud, '__init__')
    def test_get_ssh_public_key(self, mock_init, mock_generate_public_ssh_key):
        """Test oci get ssh public key method."""
        mock_init.return_value = None
        mock_generate_public_ssh_key.return_value = b'key123'

        cloud = OCICloud(**self.kwargs)
        cloud.ssh_private_key_file = 'tests/oci/api_key.pem'

        result = cloud._get_ssh_public_key()
        assert result == 'key123'

    @patch.object(OCICloud, '__init__')
    def test_get_console_log(self, mock_init):
        """Test oci get console log method."""
        mock_init.return_value = None
        cloud = OCICloud(**self.kwargs)
        cloud.get_console_log()

    @patch.object(OCICloud, '_wait_on_instance')
    @patch.object(OCICloud, '_terminate_instance')
    @patch.object(OCICloud, '_get_ssh_public_key')
    @patch.object(OCICloud, '_create_internet_gateway')
    @patch.object(OCICloud, '_create_subnet')
    @patch.object(OCICloud, '_create_vcn')
    @patch.object(OCICloud, '__init__')
    def test_launch_instance(
        self, mock_init, mock_create_vcn, mock_create_subnet,
        mock_create_ig, mock_get_ssh_key, mock_terminate_instance,
        mock_wait_on_instance
    ):
        """Test oci launch instance method."""
        mock_init.return_value = None

        logger = MagicMock()

        vcn = MagicMock()
        mock_create_vcn.return_value = vcn

        instance = MagicMock()
        instance.id = 'instance-123'

        response = MagicMock()
        response.data = instance

        client = MagicMock()
        client.launch_instance.return_value = response

        subnet = MagicMock()
        subnet.id = 'subnet-123'
        mock_create_subnet.return_value = subnet

        mock_get_ssh_key.return_value = 'key123'

        cloud = OCICloud(**self.kwargs)
        cloud.compute_client = client

        cloud.compartment_id = (
            'ocid1.compartment.oc1..'
            'aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa'
        )
        cloud.availability_domain = 'Omic:PHX-AD-1'
        cloud.timeout = 600
        cloud.subnet_id = None
        cloud.instance_type = None
        cloud.image_id = 'image123'
        cloud.logger = logger

        cloud._launch_instance()

        # Test exception case
        client.launch_instance.side_effect = Exception('Broken')
        mock_terminate_instance.side_effect = Exception(
            'Cannot terminate instance'
        )

        with pytest.raises(Exception):
            cloud._launch_instance()

    @patch.object(OCICloud, '_get_instance')
    @patch.object(OCICloud, '__init__')
    def test_set_image_id(self, mock_init, mock_get_instance):
        """Test oci set image id method."""
        mock_init.return_value = None

        instance = MagicMock()
        instance.display_name = 'instance 123'
        instance.source_details.image_id = 'image123'

        mock_get_instance.return_value = instance
        cloud = OCICloud(**self.kwargs)
        cloud._set_image_id()

    @patch.object(OCICloud, '_get_vnic_attachments')
    @patch.object(OCICloud, '_get_instance')
    @patch.object(OCICloud, '__init__')
    def test_set_instance_ip(
        self, mock_init, mock_get_instance, mock_get_vnic_attachments
    ):
        """Test oci set instance ip method."""
        mock_init.return_value = None

        instance = MagicMock()
        instance.id = 'instance123'
        instance.compartment_id = (
            'ocid1.compartment.oc1..'
            'aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa'
        )

        attachment = MagicMock()
        attachment.vnic_id = 'vnic123'

        mock_get_vnic_attachments.return_value = [attachment]

        vnic = MagicMock()
        vnic.public_ip = '0.0.0.0'
        vnic.private_ip = '10.0.0.123'

        response = MagicMock()
        response.data = vnic

        client = MagicMock()
        client.get_vnic.return_value = response

        mock_get_instance.return_value = instance

        cloud = OCICloud(**self.kwargs)
        cloud.vnet_client = client

        cloud._set_instance_ip()

    @patch.object(OCICloud, '_wait_on_instance')
    @patch.object(OCICloud, '__init__')
    def test_start_instance(self, mock_init, mock_wait_on_instance):
        """Test oci start instance method."""
        mock_init.return_value = None

        client = MagicMock()

        cloud = OCICloud(**self.kwargs)
        cloud.compute_client = client
        cloud.running_instance_id = 'instance123'
        cloud.timeout = 600

        cloud._start_instance()

    @patch.object(OCICloud, '_wait_on_instance')
    @patch.object(OCICloud, '__init__')
    def test_stop_instance(self, mock_init, mock_wait_on_instance):
        """Test oci stop instance method."""
        mock_init.return_value = None

        client = MagicMock()

        cloud = OCICloud(**self.kwargs)
        cloud.compute_client = client
        cloud.running_instance_id = 'instance123'
        cloud.timeout = 600

        cloud._stop_instance()

    @patch.object(OCICloud, '_delete_vcn')
    @patch.object(OCICloud, '_delete_internet_gateway')
    @patch.object(OCICloud, '_get_gateway_in_vcn_by_name')
    @patch.object(OCICloud, '_clear_route_rules')
    @patch.object(OCICloud, '_delete_subnet')
    @patch.object(OCICloud, '_get_subnet_by_name')
    @patch.object(OCICloud, '_get_vcn_by_name')
    @patch.object(OCICloud, '__init__')
    def test_terminate_instance(
        self, mock_init, mock_get_vcn, mock_get_subnet, mock_delete_subnet,
        mock_clear_route_rules, mock_get_gateway, mock_delete_gateway,
        mock_delete_vcn
    ):
        """Test oci terminate instance method."""
        mock_init.return_value = None

        client = MagicMock()

        vcn = MagicMock()
        vcn.id = 'vcn123'
        mock_get_vcn.return_value = vcn

        subnet = MagicMock()
        subnet.id = 'subnet123'
        mock_get_subnet.return_value = subnet

        gateway = MagicMock()
        gateway.id = 'gateway123'
        mock_get_gateway.return_value = gateway

        cloud = OCICloud(**self.kwargs)
        cloud.compute_composite_client = client
        cloud.running_instance_id = 'instance123'
        cloud.display_name = 'oci-ipa-test'
        cloud.timeout = 600

        cloud._terminate_instance()

        # No VCN
        mock_get_vcn.return_value = None
        cloud._terminate_instance()

        # Display name not created by img-proof
        cloud.display_name = 'not-ipa-test'
        cloud._terminate_instance()



import pytest
import time


def test_sles_wait_on_registration(host):
    start = time.time()
    end = start + 600

    while time.time() < end:
        result = host.run('pgrep registercloud')

        if not result.stdout.strip():
            break
        else:
            time.sleep(10)
    else:
        pytest.fail(
            'Registration did not finish properly for on-demand instance.'
        )



def test_sles_azure_infiniband(host):
    hyperv = '/var/lib/hyperv'
    hyperv_dir = host.file(hyperv)

    assert hyperv_dir.exists
    assert hyperv_dir.is_directory

    result = host.run('find {hyperv} ! -path {hyperv} | wc -l'.format(
        hyperv=hyperv
    ))
    assert int(result.stdout) > 0



# -*- coding: utf-8 -*-

"""Cloud module for testing Oracle OCI images."""

# Copyright (c) 2019 SUSE LLC. All rights reserved.
#
# This file is part of ipa. Ipa provides an api and command line
# utilities for testing images in the Public Cloud.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import oci

from img_proof import ipa_utils
from img_proof.ipa_constants import (
    OCI_DEFAULT_TYPE,
    OCI_DEFAULT_USER
)
from img_proof.ipa_exceptions import OCICloudException
from img_proof.ipa_cloud import IpaCloud


class OCICloud(IpaCloud):
    """Cloud framework class for testing Oracle OCI images."""

    def __init__(
        self,
        cleanup=None,
        config=None,
        description=None,
        distro_name=None,
        early_exit=None,
        history_log=None,
        image_id=None,
        inject=None,
        instance_type=None,
        log_level=30,
        no_default_test_dirs=False,
        cloud_config=None,
        region=None,
        results_dir=None,
        running_instance_id=None,
        ssh_private_key_file=None,
        ssh_user=None,
        subnet_id=None,
        test_dirs=None,
        test_files=None,
        timeout=None,
        collect_vm_info=None,
        compartment_id=None,
        availability_domain=None,
        signing_key_fingerprint=None,
        signing_key_file=None,
        tenancy=None,
        oci_user_id=None
    ):
        """Initialize OCI cloud framework class."""
        super(OCICloud, self).__init__(
            'oci',
            cleanup,
            config,
            description,
            distro_name,
            early_exit,
            history_log,
            image_id,
            inject,
            instance_type,
            log_level,
            no_default_test_dirs,
            cloud_config,
            region,
            results_dir,
            running_instance_id,
            test_dirs,
            test_files,
            timeout,
            collect_vm_info,
            ssh_private_key_file,
            ssh_user,
            subnet_id
        )

        self.availability_domain = (
            availability_domain or self.ipa_config['availability_domain']
        )

        if not self.availability_domain:
            raise OCICloudException(
                'Availability domain is required to connect to OCI.'
            )

        self.compartment_id = (
            compartment_id or self.ipa_config['compartment_id']
        )

        if not self.compartment_id:
            raise OCICloudException(
                'Compartment ID is required to use OCI.'
            )

        if not self.ssh_private_key_file:
            raise OCICloudException(
                'SSH private key file is required to connect to instance.'
            )

        self.ssh_user = self.ssh_user or OCI_DEFAULT_USER
        self.subnet_id = subnet_id or self.ipa_config['subnet_id']
        self.oci_user_id = oci_user_id or self.ipa_config['oci_user_id']
        self.tenancy = tenancy or self.ipa_config['tenancy']
        self.signing_key_fingerprint = (
            signing_key_fingerprint
            or self.ipa_config['signing_key_fingerprint']
        )
        self.signing_key_file = (
            signing_key_file or self.ipa_config['signing_key_file']
        )

        config = self._get_config()
        self.compute_client = oci.core.ComputeClient(config)
        self.compute_composite_client = \
            oci.core.ComputeClientCompositeOperations(
                self.compute_client
            )
        self.vnet_client = oci.core.VirtualNetworkClient(config)
        self.vnet_composite_client = \
            oci.core.VirtualNetworkClientCompositeOperations(
                self.vnet_client
            )

    def _create_internet_gateway(self, compartment_id, vcn, display_name):
        """Create gateway in VCN."""
        response = self.vnet_composite_client.create_internet_gateway_and_wait_for_state(  # noqa
            oci.core.models.CreateInternetGatewayDetails(
                display_name=display_name + '-gateway',
                compartment_id=compartment_id,
                is_enabled=True,
                vcn_id=vcn.id
            ),
            wait_for_states=[
                oci.core.models.InternetGateway.LIFECYCLE_STATE_AVAILABLE
            ]
        )
        internet_gateway = response.data

        self._add_route_rule_to_gateway(internet_gateway, vcn)

        return internet_gateway

    def _delete_internet_gateway(self, gateway_id):
        """Delete gateway matching id."""
        self.vnet_composite_client.delete_internet_gateway_and_wait_for_state(
            gateway_id,
            wait_for_states=[
                oci.core.models.InternetGateway.LIFECYCLE_STATE_TERMINATED
            ]
        )

    def _get_gateway_in_vcn_by_name(self, vcn_id, display_name):
        """
        Return gateway in VCN that matches display name.

        Return None if no gateway matches display name.
        """
        gateways = self._list_gateways_in_vcn(self.compartment_id, vcn_id)

        gateway = None
        for ig in gateways:
            if ig.display_name == display_name:
                gateway = ig
                break

        return gateway

    def _list_gateways_in_vcn(self, compartment_id, vcn_id):
        """Return a list of all gateways in VCN."""
        gateways = oci.pagination.list_call_get_all_results(
            self.vnet_client.list_internet_gateways,
            compartment_id=compartment_id,
            vcn_id=vcn_id
        ).data

        return gateways

    def _create_vcn(self, display_name, cidr_block='10.0.0.0/29'):
        """
        Create VCN in the default compartment.
        """
        response = self.vnet_composite_client.create_vcn_and_wait_for_state(
            oci.core.models.CreateVcnDetails(
                cidr_block=cidr_block,
                display_name=display_name + '-vcn',
                compartment_id=self.compartment_id
            ),
            wait_for_states=[oci.core.models.Vcn.LIFECYCLE_STATE_AVAILABLE]
        )
        vcn = response.data

        return vcn

    def _get_vcn(self, vcn_id):
        """Return VCN matching id."""
        return self.vnet_client.get_vcn(vcn_id).data

    def _get_vcn_by_name(self, vcn_name):
        """
        Return VCN that matches the display name.

        Return None if no VCNs found matching the name.
        """
        vcns = oci.pagination.list_call_get_all_results(
            self.vnet_client.list_vcns,
            compartment_id=self.compartment_id,
            display_name=vcn_name
        ).data
        return vcns[0] if vcns else None

    def _delete_vcn(self, vcn_id):
        """Delete VCN that matches id."""
        self.vnet_composite_client.delete_vcn_and_wait_for_state(
            vcn_id,
            wait_for_states=[oci.core.models.Vcn.LIFECYCLE_STATE_TERMINATED]
        )

    def _create_subnet(
        self,
        compartment_id,
        availability_domain,
        vcn,
        display_name
    ):
        """Create a new subnet in the provided VCN."""
        response = self.vnet_composite_client.create_subnet_and_wait_for_state(  # noqa
            oci.core.models.CreateSubnetDetails(
                compartment_id=compartment_id,
                availability_domain=availability_domain,
                display_name=display_name + '-subnet',
                vcn_id=vcn.id,
                cidr_block=vcn.cidr_block
            ),
            wait_for_states=[oci.core.models.Subnet.LIFECYCLE_STATE_AVAILABLE]
        )
        subnet = response.data

        return subnet

    def _get_subnet(self, subnet_id):
        """Return subnet based on id."""
        return self.vnet_client.get_subnet(subnet_id).data

    def _get_subnet_by_name(self, subnet_name, vcn_id):
        """
        Return subnet that matches the display name.

        Return None if no subnets found matching the display name.
        """
        subnets = oci.pagination.list_call_get_all_results(
            self.vnet_client.list_subnets,
            compartment_id=self.compartment_id,
            vcn_id=vcn_id,
            display_name=subnet_name
        ).data
        return subnets[0] if subnets else None

    def _delete_subnet(self, subnet_id):
        """Delete subnet given the id."""
        self.vnet_composite_client.delete_subnet_and_wait_for_state(
            subnet_id,
            wait_for_states=[oci.core.models.Subnet.LIFECYCLE_STATE_TERMINATED]
        )

    def _add_route_rule_to_gateway(self, internet_gateway, vcn):
        """Add route rule to enable SSH to instance VCN."""
        response = self.vnet_client.get_route_table(
            vcn.default_route_table_id
        )
        route_rules = response.data.route_rules

        route_rules.append(
            oci.core.models.RouteRule(
                cidr_block='0.0.0.0/0',
                network_entity_id=internet_gateway.id
            )
        )

        response = self.vnet_composite_client.update_route_table_and_wait_for_state(  # noqa
            vcn.default_route_table_id,
            oci.core.models.UpdateRouteTableDetails(route_rules=route_rules),
            wait_for_states=[
                oci.core.models.RouteTable.LIFECYCLE_STATE_AVAILABLE
            ]
        )
        route_rules = response.data

        return route_rules

    def _clear_route_rules(self, vcn):
        """Clear all route rules for the VCN."""
        self.vnet_composite_client.update_route_table_and_wait_for_state(
            vcn.default_route_table_id,
            oci.core.models.UpdateRouteTableDetails(route_rules=[]),
            wait_for_states=[
                oci.core.models.RouteTable.LIFECYCLE_STATE_AVAILABLE
            ]
        )

    def _get_config(self):
        """
        Attempt to get OCI config by file.

        If config file does not exist continue with values passed
        in to CLI and img-proof config files.
        """
        kwargs = {}
        config = {}

        if self.cloud_config:
            kwargs['file_location'] = self.cloud_config

        try:
            config = oci.config.from_file(**kwargs)
        except Exception:
            pass

        if self.oci_user_id:
            config['user'] = self.oci_user_id

        if self.signing_key_file:
            config['key_file'] = self.signing_key_file

        if self.signing_key_fingerprint:
            config['fingerprint'] = self.signing_key_fingerprint

        if self.tenancy:
            config['tenancy'] = self.tenancy

        if self.region:
            config['region'] = self.region

        return config

    def _get_instance(self):
        """Retrieve instance matching instance_id."""
        try:
            instance = self.compute_client.get_instance(
                self.running_instance_id
            ).data
        except Exception:
            raise OCICloudException(
                'Instance with ID: {instance_id} not found.'.format(
                    instance_id=self.running_instance_id
                )
            )
        return instance

    def _get_instance_state(self):
        """
        Attempt to retrieve the state of the instance.

        Raises:
            OCICloudException: If the instance cannot be found.
        """
        instance = self._get_instance()

        try:
            state = instance.lifecycle_state
        except Exception:
            raise OCICloudException(
                'Instance with id: {instance_id}, '
                'cannot be found.'.format(
                    instance_id=self.running_instance_id
                )
            )

        return state

    def _get_vnic_attachments(self, compartment_id, instance_id):
        """Return a list of vnics attached to the given instance."""
        vnic_attachments = oci.pagination.list_call_get_all_results(
            self.compute_client.list_vnic_attachments,
            compartment_id=compartment_id,
            instance_id=instance_id
        ).data

        return vnic_attachments

    def _is_instance_running(self):
        """
        Return True if instance is in running state.
        """
        return self._get_instance_state().lower() == 'running'

    def _get_ssh_public_key(self):
        """Generate SSH public key from private key."""
        key = ipa_utils.generate_public_ssh_key(self.ssh_private_key_file)
        return key.decode()

    def get_console_log(self):
        """
        Return console log output if it is available.

        Todo: The get_console_history_content method does not appear to work
              during initial implementation. Consider this in the future.
        """
        return ''

    def _launch_instance(self):
        """Launch an instance of the given image."""
        self.display_name = ipa_utils.generate_instance_name('oci-ipa-test')

        try:
            if not self.subnet_id:
                self.vcn = self._create_vcn(self.display_name)
                subnet = self._create_subnet(
                    self.compartment_id,
                    self.availability_domain,
                    self.vcn,
                    self.display_name
                )
                self._create_internet_gateway(
                    self.compartment_id,
                    self.vcn,
                    self.display_name
                )
                self.subnet_id = subnet.id

            instance_metadata = {
                'ssh_authorized_keys': self._get_ssh_public_key()
            }

            launch_instance_details = oci.core.models.LaunchInstanceDetails(
                display_name=self.display_name,
                compartment_id=self.compartment_id,
                availability_domain=self.availability_domain,
                shape=self.instance_type or OCI_DEFAULT_TYPE,
                metadata=instance_metadata,
                source_details=oci.core.models.InstanceSourceViaImageDetails(
                    image_id=self.image_id
                ),
                create_vnic_details=oci.core.models.CreateVnicDetails(
                    subnet_id=self.subnet_id
                )
            )

            response = self.compute_client.launch_instance(
                launch_instance_details
            )
            instance = response.data
        except Exception as error:
            try:
                self._terminate_instance()
            except Exception:
                pass

            if hasattr(error, 'message'):
                raise OCICloudException(error.message)
            else:
                raise
        else:
            self.running_instance_id = instance.id
            self.logger.debug('ID of instance: %s' % self.running_instance_id)
            self._wait_on_instance('RUNNING', self.timeout)

    def _set_image_id(self):
        """If existing instance used get image id."""
        instance = self._get_instance()
        self.display_name = instance.display_name
        self.image_id = instance.source_details.image_id

    def _set_instance_ip(self):
        """
        Retrieve instance ip and cache it.
        """
        instance = self._get_instance()
        self.instance_ip = None

        vnic_attachments = self._get_vnic_attachments(
            instance.compartment_id,
            instance.id
        )

        for attachment in vnic_attachments:
            vnic = self.vnet_client.get_vnic(
                attachment.vnic_id
            ).data

            public_address = vnic.public_ip
            private_address = vnic.private_ip

            if public_address or private_address:
                # Current nic has an IP address, set and finish
                self.instance_ip = public_address or private_address
                break

        if not self.instance_ip:
            raise OCICloudException(
                'IP address for instance cannot be found.'
            )

    def _start_instance(self):
        """Start the instance."""
        self.compute_client.instance_action(
            self.running_instance_id,
            'START'
        )
        self._wait_on_instance('RUNNING', self.timeout)

    def _stop_instance(self):
        """Stop the instance."""
        self.compute_client.instance_action(
            self.running_instance_id,
            'STOP'
        )
        self._wait_on_instance('STOPPED', self.timeout)

    def _terminate_instance(self):
        """Terminate the instance."""
        self.compute_composite_client.terminate_instance_and_wait_for_state(
            self.running_instance_id,
            wait_for_states=[
                oci.core.models.Instance.LIFECYCLE_STATE_TERMINATED
            ]
        )

        if not self.display_name.startswith('oci-ipa-test'):
            # Only cleanup artifacts created by img-proof
            return

        vcn = self._get_vcn_by_name(self.display_name + '-vcn')

        if not vcn:
            return

        subnet = self._get_subnet_by_name(
            self.display_name + '-subnet',
            vcn.id
        )

        if subnet:
            self._delete_subnet(subnet.id)

        # Route table references gateway
        self._clear_route_rules(vcn)

        gateway = self._get_gateway_in_vcn_by_name(
            vcn.id,
            self.display_name + '-gateway'
        )

        if gateway:
            self._delete_internet_gateway(gateway.id)

        self._delete_vcn(vcn.id)



#!/usr/bin/python3
# -*- coding: utf-8 -*-

"""img_proof cloud unit tests."""

# Copyright (c) 2019 SUSE LLC. All rights reserved.
#
# This file is part of img_proof. img_proof provides an api and command line
# utilities for testing images in the Public Cloud.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import io
import pytest
import os

from img_proof import ipa_utils
from img_proof.ipa_distro import Distro
from img_proof.ipa_exceptions import IpaCloudException, IpaSSHException
from img_proof.ipa_cloud import IpaCloud

from unittest.mock import call, MagicMock, patch
from tempfile import TemporaryDirectory

args = ['ec2']

NOT_IMPL_METHODS = [
    '_get_instance',
    '_get_instance_state',
    '_is_instance_running',
    '_launch_instance',
    '_set_image_id',
    '_set_instance_ip',
    '_start_instance',
    '_stop_instance',
    '_terminate_instance',
    'get_console_log'
]


class TestIpaCloud(object):
    """img_proof cloud test class."""

    @classmethod
    def setup_class(cls):
        """Set up temp results directory."""
        cls.results_dir = TemporaryDirectory()

    @classmethod
    def teardown_class(cls):
        """Cleanup results."""
        cls.results_dir.cleanup()

    def setup_method(self, method):
        """Set up kwargs dict."""
        self.kwargs = {
            'config': 'tests/data/config',
            'distro_name': 'SLES',
            'history_log': os.path.join(self.results_dir.name, '.history'),
            'image_id': 'fakeimage',
            'no_default_test_dirs': True,
            'results_dir': self.results_dir.name,
            'test_dirs': 'tests/data/tests',
            'test_files': ['test_image'],
            'ssh_user': 'ec2-user'
        }

    @pytest.mark.parametrize(
        "method",
        NOT_IMPL_METHODS,
        ids=NOT_IMPL_METHODS
    )
    def test_cloud_not_implemented_methods(self, method):
        """Confirm methods raise not implemented exception."""
        cloud = IpaCloud(*args, **self.kwargs)

        with pytest.raises(NotImplementedError) as error:
            getattr(cloud, method)()
        assert str(error.value) == 'Implement method in child classes.'

    def test_cloud_distro_required(self):
        """Test exception raised if no distro provided."""
        with pytest.raises(IpaCloudException) as error:
            IpaCloud(*args, config='tests/data/config')

        assert str(error.value) == \
            'Distro name is required.'

    def test_cloud_instance_image_required(self):
        """Test exception if no running instance or image id provided."""
        with pytest.raises(IpaCloudException) as error:
            IpaCloud(
                *args,
                config='tests/data/config',
                distro_name='SLES'
            )

        assert str(error.value) == \
            'Image ID or running instance is required.'

    @patch.object(ipa_utils, 'get_ssh_client')
    def test_cloud_get_ssh_client(self, mock_get_ssh_client):
        """Test get ssh client method."""
        cloud = IpaCloud(*args, **self.kwargs)

        cloud.instance_ip = '127.0.0.1'
        cloud.ssh_user = 'ec2-user'
        cloud.ssh_private_key_file = 'tests/data/ida_test'

        client = MagicMock()
        mock_get_ssh_client.return_value = client

        val = cloud._get_ssh_client()
        assert val == client
        assert mock_get_ssh_client.call_count == 1

        ipa_utils.clear_cache()

    @patch('img_proof.ipa_cloud.ipa_utils.generate_public_ssh_key')
    def test_cloud_get_user_data(self, mock_generate_ssh_key):
        mock_generate_ssh_key.return_value = b'testkey12345'

        provider = IpaCloud(*args, **self.kwargs)

        result = provider._get_user_data()

        assert result == \
            '#!/bin/bash\n' \
            'echo testkey12345 >> /home/ec2-user/.ssh/authorized_keys\n'

    def test_cloud_get_non_null_values(self):
        """Test cloud get non null values method."""
        cloud = IpaCloud(*args, **self.kwargs)

        data = {'region': 'us-east-1', 'type': None}

        # Assert arg takes precedence
        val = cloud._get_non_null_values(data)
        assert 'type' not in val
        assert val['region'] == 'us-east-1'

    def test_cloud_merge_results(self):
        """Test merge results output."""
        cloud = IpaCloud(*args, **self.kwargs)

        results = {
            "tests": [
                {"name": "img_proof/tests/test_sles.py::test_sles",
                 "teardown": {"duration": 4.792213439941406e-05,
                              "outcome": "passed",
                              "name": "teardown"},
                 "setup": {"duration": 9.799003601074219e-05,
                           "outcome": "passed",
                           "name": "setup"},
                 "run_index": 0,
                 "call": {"duration": 6.4849853515625e-05,
                          "outcome": "passed",
                          "name": "call"},
                 "duration": 0.00030875205993652344,
                 "outcome": "passed"}
            ],
            "summary": {"duration": 0.004277944564819336,
                        "passed": 1,
                        "num_tests": 1,
                        "failed": 0}
        }

        cloud._merge_results(results)
        for key, val in results['summary'].items():
            assert cloud.results['summary'][key] == val

        for key, val in results['tests'][0].items():
            assert cloud.results['tests'][0][key] == val

    def test_process_test_results(self):
        cloud = IpaCloud(*args, **self.kwargs)
        cloud._process_test_results(5.0, 'test_test')

        assert cloud.results['summary']['duration'] == 5.0
        assert cloud.results['summary']['num_tests'] == 1
        assert cloud.results['summary']['passed'] == 1

        test = cloud.results['tests'][0]
        assert test['outcome'] == 'passed'
        assert test['name'] == 'test_test'

    @patch.object(IpaCloud, '_merge_results')
    @patch.object(pytest, 'main')
    def test_cloud_run_tests(self, mock_pytest, mock_merge_results):
        """Test run tests method."""
        mock_pytest.return_value = 0
        mock_merge_results.return_value = None

        cloud = IpaCloud(*args, **self.kwargs)

        cloud.terminate = True
        cloud.results['info'] = {
            'platform': 'ec2',
            'region': 'us-west-1'
        }

        out = cloud._run_tests(
            ['tests/data/tests/test_image.py'],
            'test.ssh'
        )
        assert out == 0
        assert mock_pytest.call_count == 1
        assert mock_merge_results.call_count == 1

    def test_cloud_invalid_distro_name(self):
        """Test invalid distro name provided raises exception."""
        cloud = IpaCloud(*args, **self.kwargs)
        cloud.distro_name = 'BadDistro'

        with pytest.raises(IpaCloudException) as error:
            cloud._set_distro()
        assert str(error.value) == 'Distribution: BadDistro, not supported.'

    @patch.object(IpaCloud, '_is_instance_running')
    @patch.object(IpaCloud, '_start_instance')
    def test_cloud_start_if_stopped(
        self,
        mock_start_instance,
        mock_instance_running
    ):
        """Test start instance if stopped method."""
        mock_instance_running.return_value = False
        mock_start_instance.return_value = None

        cloud = IpaCloud(*args, **self.kwargs)
        cloud._start_instance_if_stopped()

        assert mock_instance_running.call_count == 1
        assert mock_start_instance.call_count == 1

    @patch('img_proof.ipa_utils.execute_ssh_command')
    def test_cloud_execute_ssh_command(self, mock_exec_cmd):
        client = MagicMock()
        mock_exec_cmd.return_value = 'command executed successfully!'

        cloud = IpaCloud(*args, **self.kwargs)
        cloud.log_file = 'fake_file.name'

        with patch('builtins.open', create=True) as mock_open:
            mock_open.return_value = MagicMock(spec=io.IOBase)
            file_handle = mock_open.return_value.__enter__.return_value

            cloud.execute_ssh_command(client, 'python test.py')

            file_handle.write.assert_has_calls([
                call('\n'),
                call('command executed successfully!')
            ])

    @patch('img_proof.ipa_utils.extract_archive')
    def test_cloud_extract_archive(self, mock_extract_archive):
        client = MagicMock()

        mock_extract_archive.return_value = 'archive extracted successfully!'

        cloud = IpaCloud(*args, **self.kwargs)
        cloud.log_file = 'fake_file.name'

        with patch('builtins.open', create=True) as mock_open:
            mock_open.return_value = MagicMock(spec=io.IOBase)
            file_handle = mock_open.return_value.__enter__.return_value

            cloud.extract_archive(client, 'archive.tar.xz')

            file_handle.write.assert_has_calls([
                call('\n'),
                call('archive extracted successfully!')
            ])

        mock_extract_archive.assert_called_once_with(
            client, 'archive.tar.xz', None
        )

    @patch.object(IpaCloud, '_set_instance_ip')
    @patch.object(IpaCloud, '_stop_instance')
    @patch.object(IpaCloud, '_start_instance')
    def test_cloud_hard_reboot(
        self,
        mock_start_instance,
        mock_stop_instance,
        mock_set_instance_ip
    ):
        """Test start instance if stopped method."""
        mock_stop_instance.return_value = None
        mock_start_instance.return_value = None
        mock_set_instance_ip.return_value = None

        cloud = IpaCloud(*args, **self.kwargs)
        cloud.instance_ip = '0.0.0.0'
        cloud.hard_reboot_instance()

        assert mock_stop_instance.call_count == 1
        assert mock_start_instance.call_count == 1
        assert mock_set_instance_ip.call_count == 1

    @patch('img_proof.ipa_utils.put_file')
    def test_cloud_put_file(self, mock_put_file):
        client = MagicMock()

        file_path = '/home/user/test.file'
        basename = 'test.file'

        cloud = IpaCloud(*args, **self.kwargs)
        out = cloud.put_file(client, file_path)

        assert out == basename

        mock_put_file.assert_called_once_with(
            client, file_path, basename
        )

    def test_cloud_install_package(self):
        client = MagicMock()
        distro = MagicMock()
        distro.install_package.return_value = 'package install successful!'

        cloud = IpaCloud(*args, **self.kwargs)
        cloud.log_file = 'fake_file.name'
        cloud.distro = distro

        with patch('builtins.open', create=True) as mock_open:
            mock_open.return_value = MagicMock(spec=io.IOBase)
            file_handle = mock_open.return_value.__enter__.return_value

            cloud.install_package(client, 'python')

            file_handle.write.assert_has_calls([
                call('\n'),
                call('package install successful!')
            ])

    @patch.object(IpaCloud, 'execute_ssh_command')
    @patch.object(IpaCloud, 'extract_archive')
    @patch.object(IpaCloud, 'install_package')
    @patch.object(IpaCloud, 'put_file')
    def test_process_injection_file(
        self,
        mock_put_file,
        mock_install_package,
        mock_extract_archive,
        mock_execute_command
    ):
        client = MagicMock()
        mock_put_file.side_effect = [
            'test.noarch.rpm', 'test.tar.xz', 'test.py'
        ]

        cloud = IpaCloud(*args, **self.kwargs)
        cloud.inject = 'tests/data/injection/test_injection.yaml'

        cloud.process_injection_file(client)

        mock_put_file.assert_has_calls([
            call(client, '/home/user/test.noarch.rpm'),
            call(client, '/home/user/test.tar.xz'),
            call(client, '/home/user/test.py')
        ])

        mock_install_package.assert_has_calls([
            call(client, 'test.noarch.rpm'),
            call(client, 'python3')
        ])

        mock_extract_archive.assert_called_once_with(
            client, 'test.tar.xz'
        )

        mock_execute_command.assert_called_once_with(
            client, 'python test.py'
        )

    @patch.object(IpaCloud, 'get_console_log')
    @patch.object(IpaCloud, '_set_instance_ip')
    @patch.object(IpaCloud, '_set_image_id')
    @patch.object(IpaCloud, '_start_instance_if_stopped')
    @patch.object(IpaCloud, '_get_ssh_client')
    def test_cloud_unable_connect_instance(
        self,
        mock_get_ssh_client,
        mock_start_instance,
        mock_set_image_id,
        mock_set_instance_ip,
        mock_get_console_log
    ):
        """Test exception raised when connection cannot be established."""
        mock_get_ssh_client.side_effect = IpaSSHException('ERROR!')
        mock_start_instance.return_value = None
        mock_set_image_id.return_value = None
        mock_set_instance_ip.return_value = None
        mock_get_console_log.return_value = 'Console log output...'
        self.kwargs['running_instance_id'] = 'fakeinstance'

        cloud = IpaCloud(*args, **self.kwargs)
        with pytest.raises(IpaCloudException) as error:
            cloud.test_image()
        assert str(error.value) == 'Unable to connect to instance: ERROR!'
        assert mock_get_ssh_client.call_count == 1

    @patch.object(IpaCloud, '_set_instance_ip')
    @patch.object(IpaCloud, '_set_image_id')
    @patch.object(IpaCloud, '_start_instance_if_stopped')
    @patch.object(IpaCloud, '_get_ssh_client')
    @patch('img_proof.ipa_utils.get_host_key_fingerprint')
    @patch.object(IpaCloud, 'hard_reboot_instance')
    def test_cloud_bad_connect_hard_reboot(
        self,
        mock_hard_reboot,
        mock_get_host_key,
        mock_get_ssh_client,
        mock_start_instance,
        mock_set_image_id,
        mock_set_instance_ip
    ):
        """Test exception when connection not established after hard reboot."""
        mock_hard_reboot.return_value = None
        mock_get_host_key.return_value = b'04820482'
        mock_get_ssh_client.side_effect = [None, IpaSSHException('ERROR!')]
        mock_start_instance.return_value = None
        mock_set_image_id.return_value = None
        mock_set_instance_ip.return_value = None
        self.kwargs['running_instance_id'] = 'fakeinstance'
        self.kwargs['test_files'] = ['test_hard_reboot']

        cloud = IpaCloud(*args, **self.kwargs)
        cloud.ssh_private_key_file = 'tests/data/ida_test'
        cloud.ssh_user = 'root'
        cloud.logger = MagicMock()

        cloud.test_image()
        cloud.logger.error.assert_called_once_with(
            'Unable to connect to instance after hard reboot: ERROR!'
        )
        cloud.logger.error.reset_mock()

        assert mock_get_ssh_client.call_count > 0
        assert mock_hard_reboot.call_count == 1

        mock_hard_reboot.reset_mock()
        cloud.results_dir = self.results_dir.name

        mock_get_ssh_client.side_effect = [None, Exception('ERROR!')]

        cloud.test_image()
        cloud.logger.error.assert_called_once_with(
            'Instance failed hard reboot: ERROR!'
        )

        assert mock_get_ssh_client.call_count > 0
        assert mock_hard_reboot.call_count == 1
        mock_hard_reboot.reset_mock()

    @patch.object(IpaCloud, '_set_instance_ip')
    @patch.object(IpaCloud, '_set_image_id')
    @patch.object(IpaCloud, '_start_instance_if_stopped')
    @patch.object(IpaCloud, '_get_ssh_client')
    @patch('img_proof.ipa_utils.get_host_key_fingerprint')
    @patch.object(Distro, 'reboot')
    def test_cloud_bad_connect_soft_reboot(
        self,
        mock_soft_reboot,
        mock_get_host_key,
        mock_get_ssh_client,
        mock_start_instance,
        mock_set_image_id,
        mock_set_instance_ip
    ):
        """Test exception when connection not established after hard reboot."""
        mock_soft_reboot.return_value = None
        mock_get_host_key.return_value = b'04820482'
        mock_get_ssh_client.side_effect = [
            None,
            None,
            IpaSSHException('ERROR!')
        ]
        mock_start_instance.return_value = None
        mock_set_image_id.return_value = None
        mock_set_instance_ip.return_value = None
        self.kwargs['running_instance_id'] = 'fakeinstance'
        self.kwargs['test_files'] = ['test_soft_reboot']

        cloud = IpaCloud(*args, **self.kwargs)
        cloud.ssh_private_key_file = 'tests/data/ida_test'
        cloud.ssh_user = 'root'
        cloud.logger = MagicMock()

        cloud.test_image()
        cloud.logger.error.assert_called_once_with(
            'Unable to connect to instance after soft reboot: ERROR!'
        )
        cloud.logger.error.reset_mock()

        assert mock_get_ssh_client.call_count > 0
        assert mock_soft_reboot.call_count == 1

        mock_soft_reboot.reset_mock()
        cloud.results_dir = self.results_dir.name

        mock_get_ssh_client.side_effect = [None, None, Exception('ERROR!')]

        cloud.test_image()
        cloud.logger.error.assert_called_once_with(
            'Instance failed soft reboot: ERROR!'
        )

        assert mock_get_ssh_client.call_count > 0
        assert mock_soft_reboot.call_count == 1
        mock_soft_reboot.reset_mock()

    @patch.object(IpaCloud, '_set_instance_ip')
    @patch.object(IpaCloud, '_set_image_id')
    @patch.object(IpaCloud, '_start_instance_if_stopped')
    @patch.object(IpaCloud, '_get_ssh_client')
    @patch.object(IpaCloud, '_terminate_instance')
    @patch('img_proof.ipa_utils.get_host_key_fingerprint')
    @patch.object(Distro, 'update')
    def test_cloud_distro_update(
        self,
        mock_distro_update,
        mock_get_host_key,
        mock_terminate_instance,
        mock_get_ssh_client,
        mock_start_instance,
        mock_set_image_id,
        mock_set_instance_ip
    ):
        """Test exception raised when invalid test item provided."""
        mock_distro_update.return_value = 'Updated!'
        mock_get_host_key.return_value = b'04820482'
        mock_terminate_instance.return_value = None
        mock_get_ssh_client.return_value = None
        mock_start_instance.return_value = None
        mock_set_image_id.return_value = None
        mock_set_instance_ip.return_value = None
        self.kwargs['running_instance_id'] = 'fakeinstance'
        self.kwargs['test_files'] = ['test_update']
        self.kwargs['cleanup'] = True

        cloud = IpaCloud(*args, **self.kwargs)
        cloud.ssh_private_key_file = 'tests/data/ida_test'
        cloud.ssh_user = 'root'

        status, results = cloud.test_image()
        assert status == 0
        assert mock_distro_update.call_count == 1
        self.kwargs['cleanup'] = None

    @patch.object(IpaCloud, '_set_instance_ip')
    @patch.object(IpaCloud, '_set_image_id')
    @patch.object(IpaCloud, '_start_instance_if_stopped')
    @patch.object(IpaCloud, '_get_ssh_client')
    @patch('img_proof.ipa_utils.get_host_key_fingerprint')
    @patch.object(IpaCloud, '_run_tests')
    def test_cloud_break_if_test_failure(
        self,
        mock_run_tests,
        mock_get_host_key,
        mock_get_ssh_client,
        mock_start_instance,
        mock_set_image_id,
        mock_set_instance_ip
    ):
        """Test exception raised when invalid test item provided."""
        mock_run_tests.return_value = 1
        mock_get_host_key.return_value = b'04820482'
        mock_get_ssh_client.return_value = None
        mock_start_instance.return_value = None
        mock_set_image_id.return_value = None
        mock_set_instance_ip.return_value = None
        self.kwargs['running_instance_id'] = 'fakeinstance'
        self.kwargs['early_exit'] = True

        cloud = IpaCloud(*args, **self.kwargs)
        cloud.ssh_private_key_file = 'tests/data/ida_test'
        cloud.ssh_user = 'root'

        status, results = cloud.test_image()
        assert status == 1
        assert mock_run_tests.call_count == 1

    @patch.object(IpaCloud, '_get_instance_state')
    @patch('time.sleep')
    def test_cloud_wait_on_instance(self,
                                    mock_sleep,
                                    mock_get_instance_state):
        """Test wait on instance method."""
        mock_get_instance_state.return_value = 'Stopped'
        mock_sleep.return_value = None

        cloud = IpaCloud(*args, **self.kwargs)
        cloud._wait_on_instance('Stopped')
        assert mock_get_instance_state.call_count == 1

    @patch.object(IpaCloud, '_get_ssh_client')
    def test_collect_vm_info(self, mock_get_ssh_client):
        """Test collect_vm_info method. """
        distro = MagicMock()
        client = MagicMock()
        distro.get_vm_info.return_value = \
            'Failed to collect VM info: Does not exist.'
        mock_get_ssh_client.return_value = client

        cloud = IpaCloud(*args, **self.kwargs)
        cloud.distro = distro
        cloud.log_file = 'fake_file.name'
        cloud.logger = MagicMock()

        with patch('builtins.open', create=True) as mock_open:
            mock_open.return_value = MagicMock(spec=io.IOBase)
            file_handle = mock_open.return_value.__enter__.return_value

            cloud._collect_vm_info()

            file_handle.write.assert_has_calls([
                call('\n'),
                call('Failed to collect VM info: Does not exist.')
            ])

        cloud.logger.info.assert_called_once_with(
            'Collecting basic info about VM'
        )
        assert mock_get_ssh_client.call_count == 1



#!/usr/bin/python3
# -*- coding: utf-8 -*-

"""img_proof SSH provider unit tests."""

# Copyright (c) 2019 SUSE LLC. All rights reserved.
#
# This file is part of img_proof. img_proof provides an api and command line
# utilities for testing images in the Public Cloud.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import pytest

from img_proof.ipa_exceptions import SSHCloudException
from img_proof.ipa_ssh import SSHCloud


class TestSSHProvider(object):
    """Test SSH provider class."""

    def setup_method(self, method):
        """Set up kwargs dict."""
        self.kwargs = {
            'config': 'tests/data/config',
            'distro_name': 'sles',
            'ip_address': '10.0.0.1',
            'no_default_test_dirs': True,
            'ssh_private_key_file': 'tests/data/ida_test',
            'ssh_user': 'root',
            'test_dirs': 'tests/data/tests',
            'test_files': ['test_image']
        }

    @pytest.mark.parametrize('name,msg', [
        ('ssh_user', 'SSH user is required to connect to instance.'),
        (
            'ssh_private_key_file',
            'SSH private key file is required to connect to instance.'
        ),
        ('ip_address', 'IP address is required to connect to instance.')
    ])
    def test_required_args(self, name, msg):
        self.kwargs[name] = None

        with pytest.raises(SSHCloudException) as error:
            SSHCloud(**self.kwargs)

        assert str(error.value) == msg

    @pytest.mark.parametrize('name,result', [
        ('_is_instance_running', True),
        ('_get_instance_state', None),
        ('_get_instance', None)
    ])
    def test_ssh_empty_methods(self, name, result):
        """Test empty methods."""
        provider = SSHCloud(**self.kwargs)
        output = getattr(provider, name)()

        assert output == result

    @pytest.mark.parametrize('name,msg', [
        ('_launch_instance', 'SSH class cannot launch instances.'),
        ('_set_image_id', 'SSH class has no access to cloud API.'),
        ('_set_instance_ip', 'SSH class has no access to cloud API.'),
        ('_start_instance', 'SSH class has no access to cloud API.'),
        ('_stop_instance', 'SSH class has no access to cloud API.'),
        ('_terminate_instance', 'SSH class has no access to cloud API.')
    ])
    def test_ssh_not_implemented_methods(self, name, msg):
        """Test an exception is raised for methods that are not implemeted."""
        provider = SSHCloud(**self.kwargs)

        # Assert exception is raised
        with pytest.raises(SSHCloudException) as error:
            getattr(provider, name)()

        assert str(error.value) == msg





def test_sles_ec2_billing_code(get_ec2_billing_products):
    """
    This is really ONLY for SLES.

    All other on-demand products are sold through the Marketplace
    and they do not have a billing-product-code.
    """
    assert get_ec2_billing_products()



#!/usr/bin/python3
# -*- coding: utf-8 -*-

"""img_proof ec2 provider unit tests."""

# Copyright (c) 2019 SUSE LLC. All rights reserved.
#
# This file is part of img_proof. img_proof provides an api and command line
# utilities for testing images in the Public Cloud.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import boto3
import pytest

from img_proof.ipa_ec2 import EC2Cloud
from img_proof.ipa_exceptions import EC2CloudException

from unittest.mock import MagicMock, patch


class TestEC2Provider(object):
    """Test EC2 provider class."""

    def setup_method(self, method):
        """Set up kwargs dict."""
        self.kwargs = {
            'account_name': 'bob',
            'config': 'tests/data/config',
            'distro_name': 'SLES',
            'image_id': 'fakeimage',
            'no_default_test_dirs': True,
            'cloud_config': 'tests/ec2/.ec2utils.conf',
            'test_files': ['test_image'],
            'ssh_key_name': 'test-key'
        }

    def test_ec2_exception_required_args(self):
        """Test an exception is raised if required args missing."""
        self.kwargs['config'] = 'tests/data/config.noregion'
        msg = 'Region is required to connect to EC2.'

        # Test region required
        with pytest.raises(EC2CloudException) as error:
            EC2Cloud(**self.kwargs)

        assert str(error.value) == msg

        self.kwargs['cloud_config'] = 'tests/ec2/.ec2utils.conf.nokey'
        self.kwargs['config'] = 'tests/data/config'
        msg = 'SSH private key file is required to connect to instance.'

        # Test ssh private key file required
        with pytest.raises(EC2CloudException) as error:
            EC2Cloud(**self.kwargs)

        assert str(error.value) == msg
        self.kwargs['cloud_config'] = 'tests/ec2/.ec2utils.conf'

    @patch.object(boto3, 'resource')
    def test_ec2_bad_connection(self, mock_boto3):
        """Test an exception is raised if boto3 unable to connect."""
        mock_boto3.side_effect = Exception('ERROR!')

        provider = EC2Cloud(**self.kwargs)
        msg = 'Could not connect to region: %s' % provider.region

        # Test ssh private key file required
        with pytest.raises(EC2CloudException) as error:
            provider._connect()

        assert str(error.value) == msg
        assert mock_boto3.call_count > 0

    @patch.object(EC2Cloud, '_connect')
    def test_ec2_get_instance(self, mock_connect):
        """Test get instance method."""
        instance = MagicMock()
        resource = MagicMock()
        resource.Instance.return_value = instance
        mock_connect.return_value = resource

        provider = EC2Cloud(**self.kwargs)
        val = provider._get_instance()
        assert val == instance
        assert mock_connect.call_count == 1

    @patch.object(EC2Cloud, '_connect')
    def test_ec2_get_instance_error(self, mock_connect):
        """Test get instance method error."""
        resource = MagicMock()
        resource.Instance.side_effect = Exception('Error!')
        mock_connect.return_value = resource

        provider = EC2Cloud(**self.kwargs)
        provider.running_instance_id = 'i-123456789'

        with pytest.raises(EC2CloudException) as error:
            provider._get_instance()

        msg = 'Instance with ID: i-123456789 not found.'
        assert str(error.value) == msg

    @patch.object(EC2Cloud, '_get_instance')
    def test_ec2_get_instance_state(self, mock_get_instance):
        """Test an exception is raised if boto3 unable to connect."""
        instance = MagicMock()
        instance.state = {'Name': 'ReadyRole'}
        mock_get_instance.return_value = instance

        provider = EC2Cloud(**self.kwargs)
        val = provider._get_instance_state()
        assert val == 'ReadyRole'
        assert mock_get_instance.call_count == 1

        instance.state = {}
        mock_get_instance.reset_mock()
        msg = 'Instance with id: %s, cannot be found.' \
              % provider.running_instance_id

        # Test exception raised if instance state not found
        with pytest.raises(EC2CloudException) as error:
            provider._get_instance_state()

        assert str(error.value) == msg
        assert mock_get_instance.call_count == 1

    @patch.object(EC2Cloud, '_get_instance_state')
    def test_ec2_is_instance_running(self, mock_get_instance_state):
        """Test ec2 provider is instance runnning method."""
        mock_get_instance_state.return_value = 'running'

        provider = EC2Cloud(**self.kwargs)
        assert provider._is_instance_running()
        assert mock_get_instance_state.call_count == 1

        mock_get_instance_state.return_value = 'stopped'
        mock_get_instance_state.reset_mock()

        provider = EC2Cloud(**self.kwargs)
        assert not provider._is_instance_running()
        assert mock_get_instance_state.call_count == 1

    @patch.object(EC2Cloud, '_wait_on_instance')
    @patch.object(EC2Cloud, '_connect')
    def test_ec2_launch_instance(self, mock_connect, mock_wait_on_instance):
        """Test ec2 provider launch instance method."""
        instance = MagicMock()
        instance.instance_id = 'i-123456789'
        instances = [instance]

        resource = MagicMock()
        resource.create_instances.return_value = instances

        mock_connect.return_value = resource

        provider = EC2Cloud(**self.kwargs)
        provider.subnet_id = 'subnet-123456789'
        provider.security_group_id = 'sg-123456789'
        provider._launch_instance()

        mock_wait_on_instance.assert_called_once_with('running', 600)
        assert instance.instance_id == provider.running_instance_id
        assert resource.create_instances.call_count == 1

    @patch.object(EC2Cloud, '_get_instance')
    def test_ec2_set_image_id(self, mock_get_instance):
        """Test ec2 provider set image id method."""
        instance = MagicMock()
        instance.image_id = 'ami-123456'
        mock_get_instance.return_value = instance

        provider = EC2Cloud(**self.kwargs)
        provider._set_image_id()

        assert provider.image_id == instance.image_id
        assert mock_get_instance.call_count == 1

    @patch.object(EC2Cloud, '_get_instance')
    def test_ec2_set_instance_ip(self, mock_get_instance):
        """Test ec2 provider set image id method."""
        instance = MagicMock()
        instance.public_ip_address = None
        instance.private_ip_address = None
        instance.network_interfaces = []
        mock_get_instance.return_value = instance

        provider = EC2Cloud(**self.kwargs)
        msg = 'IP address for instance cannot be found.'

        with pytest.raises(EC2CloudException) as error:
            provider._set_instance_ip()

        assert str(error.value) == msg
        assert mock_get_instance.call_count == 1
        mock_get_instance.reset_mock()

        instance.private_ip_address = '127.0.0.1'

        provider._set_instance_ip()
        assert provider.instance_ip == '127.0.0.1'
        assert mock_get_instance.call_count == 1
        mock_get_instance.reset_mock()

        network_interface = MagicMock()
        network_interface.ipv6_addresses = ['127.0.0.2']
        instance.network_interfaces = [network_interface]

        provider._set_instance_ip()
        assert provider.instance_ip == '127.0.0.2'
        assert mock_get_instance.call_count == 1
        mock_get_instance.reset_mock()

        instance.public_ip_address = '127.0.0.3'

        provider._set_instance_ip()
        assert provider.instance_ip == '127.0.0.3'
        assert mock_get_instance.call_count == 1

    @patch.object(EC2Cloud, '_wait_on_instance')
    @patch.object(EC2Cloud, '_get_instance')
    def test_ec2_start_instance(
        self, mock_get_instance, mock_wait_on_instance
    ):
        """Test ec2 start instance method."""
        instance = MagicMock()
        instance.start.return_value = None
        instance.wait_until_running.return_value = None
        mock_get_instance.return_value = instance

        provider = EC2Cloud(**self.kwargs)
        provider._start_instance()

        mock_wait_on_instance.assert_called_once_with('running', 600)
        assert mock_get_instance.call_count == 1

    @patch.object(EC2Cloud, '_wait_on_instance')
    @patch.object(EC2Cloud, '_get_instance')
    def test_ec2_stop_instance(
        self, mock_get_instance, mock_wait_on_instance
    ):
        """Test ec2 stop instance method."""
        instance = MagicMock()
        instance.stop.return_value = None
        instance.wait_until_stopped.return_value = None
        mock_get_instance.return_value = instance

        provider = EC2Cloud(**self.kwargs)
        provider._stop_instance()

        mock_wait_on_instance.assert_called_once_with('stopped', 600)
        assert mock_get_instance.call_count == 1

    @patch.object(EC2Cloud, '_get_instance')
    def test_ec2_terminate_instance(self, mock_get_instance):
        """Test ec2 terminate instance method."""
        instance = MagicMock()
        instance.terminate.return_value = None
        mock_get_instance.return_value = instance

        provider = EC2Cloud(**self.kwargs)
        provider._terminate_instance()
        assert mock_get_instance.call_count == 1

    @patch('time.sleep')
    @patch.object(EC2Cloud, '_get_instance')
    def test_ec2_get_console_log(self, mock_get_instance, mock_time):
        """Test ec2 get console log method."""
        instance = MagicMock()
        instance.console_output.return_value = {
            'Output': 'Console log output...'
        }
        mock_get_instance.return_value = instance

        provider = EC2Cloud(**self.kwargs)
        output = provider.get_console_log()
        assert output == 'Console log output...'
        assert mock_get_instance.call_count == 1



# -*- coding: utf-8 -*-

"""Ipa exceptions module."""

# Copyright (c) 2019 SUSE LLC. All rights reserved.
#
# This file is part of img_proof. img_proof provides an api and command line
# utilities for testing images in the Public Cloud.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


class IpaException(Exception):
    """Generic exception for the img_proof package."""


class IpaCloudException(IpaException):
    """Generic exception for the img_proof cloud framework."""


class AzureCloudException(IpaCloudException):
    """Generic Azure exception."""


class EC2CloudException(IpaCloudException):
    """Generic EC2 exception."""


class GCECloudException(IpaCloudException):
    """Generic GCE exception."""


class GCECloudRetryableError(GCECloudException):
    """GCE retryable error exception."""


class OCICloudException(IpaCloudException):
    """Generic OCI exception."""


class IpaControllerException(IpaException):
    """Generic exception for img_proof controller module."""


class IpaDistroException(IpaException):
    """Generic Exception for distro modules."""


class IpaImageNotFoundException(IpaException):
    """Exception for image not found on cloud framework."""


class IpaListTestsException(IpaException):
    """List subcommand exception."""


class IpaResultsException(IpaException):
    """Results subcommand exception."""


class IpaSLESException(IpaDistroException):
    """Generic Exception for distro modules."""


class IpaTestException(IpaException):
    """Test subcommand exception."""


class IpaUtilsException(IpaException):
    """Generic exception for img_proof utility methods."""


class IpaSSHException(IpaUtilsException):
    """Generic exception for img_proof SSH methods."""


class IpaRetryableError(IpaCloudException):
    """Generic retryable error exception."""


class SSHCloudException(IpaUtilsException):
    """Generic exception for SSH class."""


class LibcloudException(IpaCloudException):
    """Generic libcloud exception."""



def test_sles_motd(host, get_release_value):
    motd = host.file('/etc/motd')
    version = get_release_value('VERSION')
    assert version
    assert motd.contains(
        'SUSE Linux Enterprise Server {0}'.format(
            version.replace('-', ' ')
        )
    )



#!/usr/bin/python3
# -*- coding: utf-8 -*-

"""img_proof CLI unit tests."""

# Copyright (c) 2019 SUSE LLC. All rights reserved.
#
# This file is part of img_proof. img_proof provides an api and command line
# utilities for testing images in the Public Cloud.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os

from img_proof.scripts.cli import main

from click.testing import CliRunner

import pytest


def test_cli_ipa_help():
    """Confirm img_proof --help is successful."""
    runner = CliRunner()
    result = runner.invoke(main, ['--help'])
    assert result.exit_code == 0
    assert 'img-proof provides a Python API and cmd line utility' \
           ' for testing images.' in result.output


@pytest.mark.parametrize(
    "endpoint,value",
    [('list',
      'Print a list of test files or test cases.'),
     ('results',
      'Process provided history log and results files.'),
     ('test',
      'Test image in the given framework using the supplied test files.')],
    ids=['img_proof-list', 'img_proof-results', 'img_proof-test']
)
def test_cli_help(endpoint, value):
    """Confirm img_proof list --help is successful."""
    runner = CliRunner()
    result = runner.invoke(main, [endpoint, '--help'])
    assert result.exit_code == 0
    assert value in result.output


def test_cli_provider_missing():
    """Confirm error if provider not supplied for test endpoint."""
    runner = CliRunner()
    result = runner.invoke(main, ['test'])
    assert result.exit_code != 0
    assert 'Error: Missing argument' in result.output


def test_cli_invalid_provider():
    """Confirm error if invalid provider supplied."""
    runner = CliRunner()
    result = runner.invoke(main, ['test', 'Provider'])
    assert result.exit_code != 0
    assert 'Error: Invalid value for' in result.output


def test_cli_invalid_distro():
    """Confirm error if invalid distro provided."""
    runner = CliRunner()
    result = runner.invoke(main, ['test', '-d', 'Distro'])
    assert result.exit_code != 0
    assert 'Error: Invalid value for "-d" / "--distro"' in result.output


def test_cli_test_image_exception():
    """Confirm exception if missing image id and running instance."""
    runner = CliRunner()
    result = runner.invoke(
        main,
        ['test', 'ec2', '-d', 'SLES', '-C', 'tests/data/config']
    )
    assert result.exit_code != 0
    assert 'Image ID or running instance is required.' in result.output


def test_cli_list():
    """Test img_proof list endpoint."""
    runner = CliRunner()
    result = runner.invoke(main, ['list', 'tests/data/tests'])
    assert result.exit_code == 0


def test_cli_list_verbose():
    """Test img_proof list endpoint."""
    runner = CliRunner()
    result = runner.invoke(main, ['list', '-v', 'tests/data/tests'])
    assert result.exit_code == 0


def test_cli_results():
    """Test img_proof results endpoint."""
    runner = CliRunner()
    result = runner.invoke(
        main,
        ['results',
         '--history-log',
         'tests/data/.history',
         'show', '-v', '-r',
         'tests/data/test.results']
    )
    assert result.exit_code == 0


def test_cli_archive():
    """Test img_proof archive history sub command."""
    with open('tests/data/history.log') as f:
        log_file = f.readlines()

    with open('tests/data/history.results') as f:
        results_file = f.readlines()

    with open('tests/data/history.log') as f:
        log2_file = f.readlines()

    with open('tests/data/history.results') as f:
        results2_file = f.readlines()

    runner = CliRunner()
    with runner.isolated_filesystem():
        base_path = 'tests/data/ec2/ami-859bd1e5/i-44444444444444444/'
        os.makedirs(base_path)
        os.makedirs('archives')

        with open('tests/.history', 'w') as f:
            f.writelines([
                base_path + '20170626142856.log\n',
                base_path + '20170626142857.log\n'
            ])

        with open(base_path + '20170626142856.log', 'w') as f:
            f.writelines(log_file)

        with open(base_path + '20170626142856.results', 'w') as f:
            f.writelines(results_file)

        with open(base_path + '20170626142857.log', 'w') as f:
            f.writelines(log2_file)

        with open(base_path + '20170626142857.results', 'w') as f:
            f.writelines(results2_file)

        result = runner.invoke(
            main,
            [
                'results', '--history-log', 'tests/.history', 'archive',
                '--clear-log', 'archives/', 'archive_123'
            ]
        )
        output = result.output.split(':')[-1].strip()
        assert os.path.exists(output)

        result = runner.invoke(
            main,
            ['results', '--history-log', 'tests/.history', 'list']
        )
        assert 'Path "tests/.history" does not exist.' in result.output


def test_cli_archive_item():
    """Test img_proof archive specific history item."""
    with open('tests/data/history.log') as f:
        log_file = f.readlines()

    with open('tests/data/history.results') as f:
        results_file = f.readlines()

    runner = CliRunner()
    with runner.isolated_filesystem():
        base_path = 'tests/data/ec2/ami-859bd1e5/i-44444444444444444/'
        os.makedirs(base_path)
        os.makedirs('archives')

        with open('tests/.history', 'w') as f:
            f.writelines([
                base_path + '20170626142856.log\n',
                base_path + '20170626142857.log\n'
            ])

        with open(base_path + '20170626142856.log', 'w') as f:
            f.writelines(log_file)

        with open(base_path + '20170626142856.results', 'w') as f:
            f.writelines(results_file)

        result = runner.invoke(
            main,
            [
                'results', '--history-log', 'tests/.history', 'archive',
                '--clear-log', '-i', '2', 'archives/', 'archive_123'
            ]
        )

        output = result.output.split(':')[-1].strip()
        assert os.path.exists(output)

        result = runner.invoke(
            main,
            ['results', '--history-log', 'tests/.history', 'list']
        )
        expected_output = '1 tests/data/ec2/ami-859bd1e5/' \
            'i-44444444444444444/20170626142857.log\n'
        assert expected_output == result.output


def test_cli_archive_missing_item():
    """Test img_proof archive missing history item."""
    runner = CliRunner()
    with runner.isolated_filesystem():
        base_path = 'tests/data/ec2/ami-859bd1e5/i-44444444444444444/'
        os.makedirs('archives')
        os.makedirs('tests')

        with open('tests/.history', 'w') as f:
            f.writelines([
                base_path + '20170626142856.log\n',
            ])

        result = runner.invoke(
            main,
            [
                'results', '--history-log', 'tests/.history', 'archive',
                '--clear-log', '-i', '1', 'archives/', 'archive_123'
            ]
        )

        assert result.output == \
            "Unable to archive history item: [Errno 2] No such file or " \
            "directory: 'tests/data/ec2/ami-859bd1e5/i-44444444444444444/" \
            "20170626142856.log'\n"


def test_cli_results_log():
    """Test img_proof results log display."""
    runner = CliRunner()
    result = runner.invoke(
        main,
        ['results',
         '--history-log',
         'tests/data/.history',
         'show', '-l', '-r',
         'tests/data/history.log']
    )
    assert result.exit_code == 0


def test_cli_results_show_exception():
    """Test img_proof results show handles exception."""
    runner = CliRunner()
    result = runner.invoke(
        main,
        ['results', '--history-log', 'tests/data/.history', 'show', '100']
    )
    assert result.exit_code != 0
    assert 'History result at index 100 does not exist.' in result.output


def test_cli_results_history():
    """Test img_proof results history."""
    runner = CliRunner()
    result = runner.invoke(
        main,
        ['results', '--history-log', 'tests/data/.history', 'list']
    )
    assert result.exit_code == 0


def test_cli_results_history_exception():
    """Test img_proof results history handles exception if file missing."""
    runner = CliRunner()
    result = runner.invoke(
        main,
        ['results', '--history-log', 'tests/data/.history', 'show', '3']
    )
    assert result.exit_code != 0
    assert 'Unable to process results file:' in result.output


def test_cli_results_history_log_exception():
    """Test img_proof results history handles exception if key missing."""
    runner = CliRunner()
    result = runner.invoke(
        main,
        ['results',
         '--history-log',
         'tests/data/.history',
         'show', '-l', '3']
    )
    assert result.exit_code != 0
    assert 'Unable to open results log file' in result.output


def test_cli_results_history_log():
    """Test img_proof results history."""
    runner = CliRunner()
    result = runner.invoke(
        main,
        ['results', '--history-log', 'tests/data/.history', 'show', '-l']
    )
    assert result.exit_code == 0


def test_cli_history():
    """Test img_proof list history sub command."""
    runner = CliRunner()
    result = runner.invoke(
        main,
        ['results', '--history-log', 'tests/data/.history', 'list']
    )
    assert result.exit_code == 0


def test_cli_clear_history():
    """Test img_proof clear history sub command."""
    lines = ''
    with open('tests/data/.history') as history_log:
        lines = history_log.readlines()

    runner = CliRunner()
    with runner.isolated_filesystem():
        try:
            os.makedirs('test')
        except OSError:
            pass

        with open('test/.history', 'w') as history_log:
            history_log.writelines(lines)

        result = runner.invoke(
            main,
            ['results', '--history-log', 'test/.history', 'clear']
        )
        assert result.exit_code == 0

        result = runner.invoke(
            main,
            ['results', '--history-log', 'test/.history', 'list']
        )
        assert 'Path "test/.history" does not exist.' in result.output


def test_cli_delete_history_item():
    """Test img_proof delete history item at given index."""
    lines = []
    with open('tests/data/.history') as history_log:
        lines = history_log.readlines()

    before_count = len(lines)
    runner = CliRunner()
    with runner.isolated_filesystem():
        try:
            os.makedirs('test')
        except OSError:
            pass

        with open('test/.history', 'w') as history_log:
            history_log.writelines(lines)

        result = runner.invoke(
            main,
            ['results', '--history-log', 'test/.history', 'delete', '1']
        )
        assert result.exit_code == 0

        with open('test/.history') as history_log:
            lines = history_log.readlines()
        assert before_count - 1 == len(lines)


def test_cli_history_empty():
    """Test img_proof history endpoint with empty history log."""
    runner = CliRunner()
    result = runner.invoke(
        main,
        ['results', '--history-log', 'tests/data/empty.history', 'list']
    )
    assert result.exit_code == 0


def test_cli_results_non_json():
    """Test img_proof results non json file provided."""
    runner = CliRunner()
    result = runner.invoke(
        main,
        ['results',
         '--history-log',
         'tests/data/.history',
         'show', '-v', '-r',
         'tests/data/config']
    )
    assert result.exit_code == 1
    assert result.output.strip() == \
        'The results file is not the proper json format.'


def test_cli_results_missing_key():
    """Test img_proof results json file missing key."""
    runner = CliRunner()
    result = runner.invoke(
        main,
        ['results',
         '--history-log',
         'tests/data/.history',
         'show', '-v', '-r',
         'tests/data/bad.results']
    )
    assert result.exit_code == 1
    assert result.output.strip() == \
        "The results json is missing key: 'summary'"


def test_cli_license():
    """Test img_proof cli license."""
    runner = CliRunner()
    result = runner.invoke(
        main,
        ['--license']
    )
    assert result.exit_code == 0
    assert result.output.strip() == 'GPLv3+'



#!/usr/bin/python3
# -*- coding: utf-8 -*-

"""img_proof GCE cloud unit tests."""

# Copyright (c) 2019 SUSE LLC. All rights reserved.
#
# This file is part of img_proof. img_proof provides an api and command line
# utilities for testing images in the Public Cloud.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import pytest

from img_proof.ipa_gce import GCECloud
from img_proof.ipa_exceptions import GCECloudException

from unittest.mock import MagicMock, patch

from libcloud.common.google import ResourceNotFoundError


class TestGCECloud(object):
    """Test GCE cloud class."""

    @patch.object(GCECloud, '_validate_region')
    @patch('libcloud.compute.drivers.gce.GCENodeDriver')
    def setup(
        self,
        mock_node_driver,
        mock_validate_region
    ):
        """Set up kwargs dict."""
        self.kwargs = {
            'config': 'tests/data/config',
            'distro_name': 'SLES',
            'image_id': 'fakeimage',
            'no_default_test_dirs': True,
            'service_account_file': 'tests/gce/service-account.json',
            'ssh_private_key_file': 'tests/data/ida_test',
            'test_files': ['test_image']
        }

        driver = MagicMock()
        mock_node_driver.return_value = driver

        self.cloud = GCECloud(**self.kwargs)

    def test_gce_exception_required_args(self):
        """Test an exception is raised if required args missing."""
        self.kwargs['service_account_file'] = None
        self.kwargs['ssh_private_key_file'] = None

        # Test service account file required
        with pytest.raises(GCECloudException) as error:
            GCECloud(**self.kwargs)

        assert str(error.value) == \
            'Service account file is required to connect to GCE.'

        self.kwargs['service_account_file'] = 'tests/gce/service-account.json'

        # Test ssh private key file required
        with pytest.raises(GCECloudException) as error:
            GCECloud(**self.kwargs)

        assert str(error.value) == \
            'SSH private key file is required to connect to instance.'

        self.kwargs['ssh_private_key_file'] = 'tests/data/ida_test'

    def test_gce_get_service_account_info(self):
        """Test get service account info method."""
        self.cloud._get_service_account_info()

        assert self.cloud.service_account_email == \
            'test@test.iam.gserviceaccount.com'
        assert self.cloud.service_account_project == 'test'

    def test_gce_get_service_account_info_invalid(self):
        """Test get service account info method."""
        self.cloud.service_account_file = \
            'tests/gce/invalid-service-account.json'

        with pytest.raises(GCECloudException) as error:
            self.cloud._get_service_account_info()

        msg = 'Service account JSON file is invalid for GCE. ' \
            'client_email key is expected. See getting started ' \
            'docs for information on GCE configuration.'
        assert str(error.value) == msg

    def test_gce_get_instance(self):
        """Test gce get instance method."""
        instance = MagicMock()
        self.cloud.compute_driver.ex_get_node.return_value = instance

        val = self.cloud._get_instance()

        assert val == instance

        self.cloud.running_instance_id = 'test-instance'
        self.cloud.compute_driver.ex_get_node.side_effect = \
            ResourceNotFoundError(
                'Broken',
                'test',
                'test'
            )

        with pytest.raises(GCECloudException) as error:
            self.cloud._get_instance()

        assert str(error.value) == "Instance with id: test-instance cannot" \
            " be found: 'Broken'"

    def test_gce_get_subnet(self):
        """Test GCE get subnetwork method."""
        subnetwork = MagicMock()
        self.cloud.compute_driver.ex_get_subnetwork.return_value = subnetwork

        self.cloud.region = 'us-west-1a'
        result = self.cloud._get_subnet('test-subnet')

        assert result == subnetwork

    def test_gce_get_subnet_exception(self):
        """Test GCE get subnetwork method."""
        self.cloud.compute_driver.ex_get_subnetwork.side_effect = Exception(
            'Cannot find subnet!'
        )

        self.cloud.region = 'us-west-1a'

        msg = 'GCE subnet: test-subnet not found.'
        with pytest.raises(GCECloudException) as error:
            self.cloud._get_subnet('test-subnet')

        assert msg == str(error.value)

    @patch.object(GCECloud, '_get_subnet')
    @patch('img_proof.ipa_utils.generate_instance_name')
    def test_gce_launch_instance(
        self,
        mock_generate_instance_name,
        mock_get_subnet
    ):
        """Test GCE launch instance method."""
        instance = MagicMock()
        self.cloud.compute_driver.create_node.return_value = instance
        self.cloud.compute_driver.wait_until_running.return_value = None
        mock_generate_instance_name.return_value = 'test-instance'

        self.cloud.region = 'us-west1-a'
        self.cloud.subnet_id = 'test-subnet'

        subnet = MagicMock()
        network = MagicMock()
        subnet.network = network
        mock_get_subnet.return_value = subnet

        self.cloud._launch_instance()

        assert self.cloud.running_instance_id == 'test-instance'

    @patch.object(GCECloud, '_get_instance')
    def test_gce_set_image_id(self, mock_get_instance):
        """Test gce cloud set image id method."""
        instance = MagicMock()
        instance.image = 'test-image'
        mock_get_instance.return_value = instance

        self.cloud._set_image_id()

        assert self.cloud.image_id == instance.image
        assert mock_get_instance.call_count == 1

    @patch.object(GCECloud, '_get_driver')
    def test_gce_validate_region(self, mock_get_driver):
        """Test gce cloud set image id method."""
        driver = MagicMock()
        driver.ex_get_zone.return_value = None
        mock_get_driver.return_value = driver

        with pytest.raises(GCECloudException) as error:
            GCECloud(**self.kwargs)

        assert str(error.value) == \
            'Zone is required for GCE cloud framework: Example: us-west1-a'

        self.kwargs['region'] = 'fake'

        with pytest.raises(GCECloudException) as error:
            GCECloud(**self.kwargs)

        driver.ex_get_zone.assert_called_once_with('fake')

        assert str(error.value) == \
            'fake is not a valid GCE zone. Example: us-west1-a'

    @patch.object(GCECloud, '_get_instance')
    def test_gce_get_instance_state(self, mock_get_instance):
        """Test gce get instance method."""
        instance = MagicMock()
        instance.state = 'running'
        mock_get_instance.return_value = instance

        val = self.cloud._get_instance_state()

        assert val == 'running'
        assert mock_get_instance.call_count == 1

    @patch.object(GCECloud, '_get_instance_state')
    def test_gce_is_instance_running(self, mock_get_instance_state):
        """Test gce cloud is instance runnning method."""
        mock_get_instance_state.return_value = 'running'

        assert self.cloud._is_instance_running()
        assert mock_get_instance_state.call_count == 1

        mock_get_instance_state.return_value = 'stopped'
        mock_get_instance_state.reset_mock()

        assert not self.cloud._is_instance_running()
        assert mock_get_instance_state.call_count == 1

    @patch.object(GCECloud, '_get_instance')
    def test_gce_set_instance_ip(self, mock_get_instance):
        """Test gce cloud set instance ip method."""
        instance = MagicMock()
        instance.public_ips = []
        instance.private_ips = []
        mock_get_instance.return_value = instance

        self.cloud.running_instance_id = 'test'

        with pytest.raises(GCECloudException) as error:
            self.cloud._set_instance_ip()

        assert str(error.value) == \
            'IP address for instance: test cannot be found.'
        assert mock_get_instance.call_count == 1

        mock_get_instance.reset_mock()

        instance.public_ips = ['127.0.0.1']
        self.cloud._set_instance_ip()

        assert self.cloud.instance_ip == '127.0.0.1'
        assert mock_get_instance.call_count == 1

    @patch.object(GCECloud, '_get_instance')
    def test_gce_start_instance(self, mock_get_instance):
        """Test gce start instance method."""
        instance = MagicMock()
        mock_get_instance.return_value = instance
        self.cloud.compute_driver.ex_start_node.return_value = None
        self.cloud.compute_driver.wait_until_running.return_value = None

        self.cloud._start_instance()

        assert mock_get_instance.call_count == 1
        assert self.cloud.compute_driver.ex_start_node.call_count == 1
        assert self.cloud.compute_driver.wait_until_running.call_count == 1

    @patch.object(GCECloud, '_wait_on_instance')
    @patch.object(GCECloud, '_get_instance')
    def test_gce_stop_instance(
        self,
        mock_get_instance,
        mock_wait_on_instance
    ):
        """Test gce stop instance method."""
        instance = MagicMock()
        mock_get_instance.return_value = instance
        mock_wait_on_instance.return_value = None
        self.cloud.compute_driver.ex_stop_node.return_value = None

        self.cloud._stop_instance()

        assert mock_get_instance.call_count == 1
        assert self.cloud.compute_driver.ex_stop_node.call_count == 1

    @patch.object(GCECloud, '_get_instance')
    def test_gce_terminate_instance(self, mock_get_instance):
        """Test gce terminate instance method."""
        instance = MagicMock()
        instance.destroy.return_value = None
        mock_get_instance.return_value = instance

        self.cloud._terminate_instance()
        assert instance.destroy.call_count == 1

    @patch.object(GCECloud, '_get_instance')
    def test_gce_get_console_log(self, mock_get_instance):
        """Test gce get console log method."""
        instance = MagicMock()
        mock_get_instance.return_value = instance
        self.cloud.compute_driver.ex_get_serial_output.return_value = 'output'

        output = self.cloud.get_console_log()
        assert output == 'output'
        assert self.cloud.compute_driver.ex_get_serial_output.call_count == 1
